#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Optimization;
using System;

namespace HeuristicLab.Problems.DataAnalysis {
  /// <summary>
  /// Represents discriminant function classification data analysis models.
  /// </summary>
  [StorableClass]
  [Item("DiscriminantFunctionClassificationModel", "Represents a classification model that uses a discriminant function and classification thresholds.")]
  public class DiscriminantFunctionClassificationModel : NamedItem, IDiscriminantFunctionClassificationModel {
    [Storable]
    private IRegressionModel model;
    [Storable]
    private double[] classValues;
    // class values are not necessarily sorted in ascending order
    public IEnumerable<double> ClassValues {
      get { return (double[])classValues.Clone(); }
      set {
        if (value == null) throw new ArgumentException();
        double[] newValue = value.ToArray();
        if (newValue.Length != classValues.Length) throw new ArgumentException();
        classValues = newValue;
      }
    }
    [Storable]
    private double[] thresholds;
    public IEnumerable<double> Thresholds {
      get { return (IEnumerable<double>)thresholds.Clone(); }
      set {
        thresholds = value.ToArray();
        OnThresholdsChanged(EventArgs.Empty);
      }
    }


    [StorableConstructor]
    protected DiscriminantFunctionClassificationModel(bool deserializing) : base(deserializing) { }
    protected DiscriminantFunctionClassificationModel(DiscriminantFunctionClassificationModel original, Cloner cloner)
      : base(original, cloner) {
      model = cloner.Clone(original.model);
      classValues = (double[])original.classValues.Clone();
      thresholds = (double[])original.thresholds.Clone();
    }
    public DiscriminantFunctionClassificationModel(IRegressionModel model, IEnumerable<double> classValues, IEnumerable<double> thresholds)
      : base() {
      this.name = ItemName;
      this.description = ItemDescription;
      this.model = model;
      this.classValues = classValues.ToArray();
      this.thresholds = thresholds.ToArray();
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new DiscriminantFunctionClassificationModel(this, cloner);
    }

    public IEnumerable<double> GetEstimatedValues(Dataset dataset, IEnumerable<int> rows) {
      return model.GetEstimatedValues(dataset, rows);
    }

    public IEnumerable<double> GetEstimatedClassValues(Dataset dataset, IEnumerable<int> rows) {
      foreach (var x in GetEstimatedValues(dataset, rows)) {
        int classIndex = 0;
        // find first threshold value which is larger than x => class index = threshold index + 1
        for (int i = 0; i < thresholds.Length; i++) {
          if (x > thresholds[i]) classIndex++;
          else break;
        }
        yield return classValues.ElementAt(classIndex);
      }
    }
    #region events
    public event EventHandler ThresholdsChanged;
    protected virtual void OnThresholdsChanged(EventArgs e) {
      var listener = ThresholdsChanged;
      if (listener != null) listener(this, e);
    }
    #endregion
  }
}
